@@ -1,3 +1,4 @@
+/* -*- mode: c; c-style: k&r; c-basic-offset: 8 -*- */
 /* 
  *  Copyright (C) 2002  Ricardo Fernndezs Pascual <ric@users.sourceforge.net>
  *
@@ -21,18 +22,17 @@
 #endif
 
 #include "bookmarks-tree-model.h"
+#include "galeon-debug.h"
+#include "galeon-dnd.h"
 #include <gtk/gtktreednd.h>
 #include <glib-object.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
 #define DEBUG_PATH(path) \
 	{ \
 		gchar *s_patch = gtk_tree_path_to_string (path); \
-		g_print (#path " = %s\n", s_patch); \
+		LOG (#path " = %s\n", s_patch); \
 		g_free (s_patch); \
 	}
 
@@ -446,7 +446,7 @@ gb_tree_model_finalize_impl (GObject *object)
 	GSList *l;
 	GbTreeModel *m = GB_TREE_MODEL (object);
 	
-	DEBUG_MSG (("Finalizing a GbTreeModel\n"));
+	LOG ("Finalizing a GbTreeModel");
 
 	if (m->set)
 	{
@@ -761,8 +761,8 @@ gb_tree_model_iter_children (GtkTreeModel *tree_model,
 			     GtkTreeIter *parent)
 {
 	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
-	GbBookmark *c1;
-	int rootindex;
+	GbBookmark *c1 = NULL;
+	int rootindex = 0;
 
 	g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
 	g_return_val_if_fail (parent == NULL || parent->stamp == btree_model->stamp, FALSE);
@@ -773,7 +773,6 @@ gb_tree_model_iter_children (GtkTreeModel *tree_model,
 		{
 			GSList *li;
 			rootindex = 0;
-			c1 = NULL;
 			for (li = btree_model->roots; li; li = li->next)
 			{
 				if (GB_IS_FOLDER (li->data))
@@ -787,14 +786,14 @@ gb_tree_model_iter_children (GtkTreeModel *tree_model,
 		else
 		{
 			c1 = btree_model->roots->data;
-			rootindex = 0;
 		}
 	}
-	else
+	else if (GB_IS_FOLDER (parent->user_data))
 	{
-		GbFolder *p = parent->user_data;
+		GbFolder *p = GB_FOLDER (parent->user_data);
 		rootindex = GPOINTER_TO_INT (parent->user_data2);
-		if (!gb_bookmark_is_alias (p) || gb_tree_model_get_nth_root (btree_model, rootindex) == (GbBookmark *) p)
+		if (!gb_bookmark_is_alias (p) ||
+                    gb_tree_model_get_nth_root (btree_model, rootindex) == (GbBookmark *) p)
 		{
 			if (btree_model->only_folders)
 			{
@@ -805,10 +804,6 @@ gb_tree_model_iter_children (GtkTreeModel *tree_model,
 				c1 = p->child;
 			}
 		}
-		else 
-		{
-			c1 = NULL;
-		}
 	}
 	
 	if (c1)
@@ -890,22 +885,21 @@ gb_tree_model_iter_n_children (GtkTreeModel *tree_model,
 			       GtkTreeIter *iter)
 {
 	GbTreeModel *btree_model = (GbTreeModel *) tree_model;
-	int rootindex;
 
 	g_return_val_if_fail (GB_IS_TREE_MODEL (tree_model), 0);
 	g_return_val_if_fail (iter == NULL || iter->user_data != NULL, 0);
 	
-	rootindex = GPOINTER_TO_INT (iter->user_data2);
-
 	if (iter == NULL || iter->user_data == GB_TREE_MODEL (tree_model)->roots)
 	{
 		if (btree_model->hide_roots)
 		{
 			GSList *l;
 			int nch = 0;
+			int rootindex = 0;
 			for (l = btree_model->roots; l; l = l->next)
 			{
 				nch += gb_tree_model_bookmark_n_children (btree_model, l->data, rootindex);
+				++rootindex;
 			}
 			return nch;
 		}
@@ -933,6 +927,7 @@ gb_tree_model_iter_n_children (GtkTreeModel *tree_model,
 	}
 	else 
 	{
+		int rootindex = GPOINTER_TO_INT (iter->user_data2);
 		return gb_tree_model_bookmark_n_children (btree_model, iter->user_data, rootindex);
 	}
 }
@@ -1179,6 +1174,20 @@ gb_tree_model_drag_data_get (GtkTreeDragSource *drag_source,
 	return FALSE;
 }
 
+typedef struct 
+{
+	GbBookmarkSet *set;
+	GSList *list;
+} GbTreeModelDragDataReceivedInfo;
+
+static void 
+gb_tree_model_drag_data_received_aux (const char *url, const char *title,
+				      gpointer data)
+{
+	GbTreeModelDragDataReceivedInfo *i = data;
+	GbSite *s = gb_site_new (i->set, title, url);
+	i->list = g_slist_prepend (i->list, s);
+}
 
 static gboolean
 gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
@@ -1188,13 +1197,14 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 	GbTreeModel *gtm;
 	GtkTreeModel *src_model = NULL;
 	GtkTreePath *src_path = NULL;
-	GbBookmark *b = NULL;
+	GSList *blist = NULL;
+	const GSList *li = NULL;
 	GbBookmark *bparent;
 	gint bindex;
 	
 	g_return_val_if_fail (GB_IS_TREE_MODEL (drag_dest), FALSE);
 
-	DEBUG_MSG (("in gb_tree_model_drag_data_received\n"));
+	LOG ("in gb_tree_model_drag_data_received");
 
 	gtm = GB_TREE_MODEL (drag_dest);
 	
@@ -1203,6 +1213,7 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 					&src_model,
 					&src_path))
 	{
+		GbBookmark *b = NULL;
 		/* in the same process */
 
 		/* Copy the given row to a new position */
@@ -1228,14 +1239,38 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 			b = gb_bookmark_copy (srcb);
 		}
 		gtk_tree_path_free (src_path);
+
+		if (b)
+		{
+			blist = g_slist_prepend (blist, b);
+		}
 	}
 	else
 	{
-		/* import XBEL */
-		NOT_IMPLEMENTED;
+		GbTreeModelDragDataReceivedInfo data;
+		guint info;
+
+		data.list = NULL;
+		data.set = gb_tree_model_get_set (gtm);
+
+		if (selection_data->target == gdk_atom_intern (GALEON_DND_URL_TYPE, FALSE))
+		{
+			info = GALEON_DND_URL;
+		}
+		else if (selection_data->target == gdk_atom_intern (GALEON_DND_URI_LIST_TYPE, FALSE))
+		{
+			info = GALEON_DND_URI_LIST;
+		}
+		else
+		{
+			return FALSE;
+		}
+		galeon_dnd_drag_data_receive (NULL, NULL, -1, -1, selection_data, 
+					      info, 0, &data, gb_tree_model_drag_data_received_aux);
+		blist = g_slist_reverse (data.list);
 	}
 
-	if (!b)
+	if (!blist)
 	{
 		return FALSE;
 	}
@@ -1272,7 +1307,7 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 	{
 		
 		bparent = gb_tree_model_get_first_root (gtm);
-		DEBUG_MSG (("Defaulting the parent to the first root...\n"));
+		LOG ("Defaulting the parent to the first root...");
 
 		/* this is broken, specially when hide roots is TRUE */
 	}
@@ -1316,13 +1351,64 @@ gb_tree_model_drag_data_received (GtkTreeDragDest *drag_dest,
 		bindex = realindex;
 	}
 
-	DEBUG_MSG (("will insert into %s (at %d)\n", bparent->name, bindex));
+	LOG ("will insert into %s (at %d)", bparent->name, bindex);
 
-	gb_folder_add_child (GB_FOLDER (bparent), b, bindex);
+	for (li = blist; li; li = li->next)
+	{
+		GbBookmark *b = GB_BOOKMARK (li->data);
+		gb_folder_add_child (GB_FOLDER (bparent), b, bindex);
+		g_object_unref (b);
+		++bindex;
+	}
+	g_slist_free (blist);
 
 	return TRUE;
 }
 
+
+static gboolean 
+gb_tree_model_row_drop_possible_aux (GbTreeModel *drag_dest, GtkTreePath *path)
+{
+	GtkTreeIter iter;
+
+	/* DEBUG_PATH (path); */
+
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_dest), &iter, path))
+	{
+		GbBookmark *b = gb_tree_model_bookmark_from_iter (GB_TREE_MODEL (drag_dest), &iter);
+		if (b)
+		{
+			GbFolder *parent = b->parent;
+			LOG ("b = %s\n", b->name);
+			LOG ("parent = %s\n", parent ? GB_BOOKMARK (parent)->name : "(null)");
+
+			return parent != NULL && !gb_folder_is_autogenerated (parent);
+		}
+	}
+
+	/* try the parent */
+	{
+		GtkTreePath *tmp = gtk_tree_path_copy (path);
+		gboolean ret = FALSE;
+		if (gtk_tree_path_up (tmp))
+		{
+			/* DEBUG_PATH (tmp); */
+			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_dest), &iter, tmp))
+			{
+				GbBookmark *p = gb_tree_model_bookmark_from_iter (GB_TREE_MODEL (drag_dest), &iter);
+				LOG ("p = %s\n", p ? p->name : "(null)");
+				if (p && GB_IS_FOLDER (p))
+				{
+					ret = !gb_folder_is_autogenerated (GB_FOLDER (p));
+				}
+			}
+		}
+		gtk_tree_path_free (tmp);
+		return ret;
+	}
+}
+
+
 static gboolean
 gb_tree_model_row_drop_possible (GtkTreeDragDest *drag_dest,
 				 GtkTreePath *dest_path,
@@ -1330,7 +1416,6 @@ gb_tree_model_row_drop_possible (GtkTreeDragDest *drag_dest,
 {
 	GtkTreeModel *src_model = NULL;
 	GtkTreePath *src_path = NULL;
-	GtkTreePath *tmp = NULL;
 	gboolean retval = TRUE;
 	
 	if (gtk_tree_get_row_drag_data (selection_data,
@@ -1364,33 +1449,10 @@ gb_tree_model_row_drop_possible (GtkTreeDragDest *drag_dest,
 	if (retval)
 	{
 		/* Can't drop if dest_path's parent doesn't exist and is a folder */
-
-		GtkTreeIter iter;
-		
-		if (gtk_tree_path_get_depth (dest_path) > 1)
-		{
-			tmp = gtk_tree_path_copy (dest_path);
-			gtk_tree_path_up (tmp);
-			
-			if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_dest),
-						      &iter, tmp))
-			{
-				retval = GB_IS_FOLDER (gb_tree_model_bookmark_from_iter 
-						       (GB_TREE_MODEL (drag_dest), &iter));
-			}
-			else
-			{
-				retval = TRUE;
-			}
-			gtk_tree_path_free (tmp);
-		}
-		else
-		{
-			retval = TRUE;
-		}
+		retval = gb_tree_model_row_drop_possible_aux (GB_TREE_MODEL (drag_dest), dest_path);
 	}
 	
-	DEBUG_MSG (("gb_tree_model_row_drop_possible () = %s\n", retval ? "TRUE" : "FALSE"));
+	LOG ("gb_tree_model_row_drop_possible () = %s", retval ? "TRUE" : "FALSE");
 	
 	return retval;
 	
@@ -1399,7 +1461,7 @@ gb_tree_model_row_drop_possible (GtkTreeDragDest *drag_dest,
 static GbBookmark *
 gb_tree_model_get_nth_root (GbTreeModel *m, int i)
 {
-	GSList *sli = g_slist_nth (m->roots, i);
+        GSList *sli = g_slist_nth (m->roots, i);
 	return sli ? sli->data : NULL;
 }
 
@@ -1442,7 +1504,7 @@ gb_tree_model_iter_from_bookmark (GbTreeModel *model, GbBookmark *b, GtkTreeIter
 {
 	int rootindex = 0;
 	GSList *sli;
-	gboolean ret;
+	gboolean ret = FALSE;
 
 	if (model->only_folders && !GB_IS_FOLDER (b))
 	{
