@@ -32,6 +32,20 @@ pref("view_source.wrap_long_lines", true);
 // disable forms manager while we have no UI
 pref("wallet.captureForms", false);
 
+// disable SSLv2, it has security issues
+pref("security.enable_ssl2", false);
+
+// set mozilla's locale to match the system (firefox/thunderbird only)
+pref("intl.locale.matchOS", true);
+
+// some distros apply a patch to firefox to allow typeaheadfind to work
+// It's a hack, but we turn it on, in case it's there - otherwise this does nothing
+// http://bugzilla.gnome.org/show_bug.cgi?id=157435
+pref("accessibility.typeaheadfindsea", true);
+
+// typeaheadfind won't handle spaces if autostart is off
+pref("accessibility.typeaheadfindsea.autostart", true);
+
 //===========================================================================
 // You should not change the rest of the preferences below.
 
@@ -47,9 +61,6 @@ pref("xpinstall.enabled", false);
 // disable sucky XUL ftp view, have nice ns4-like html page instead
 pref("network.dir.generate_html", true);
 
-// set the right accept encoding flags
-pref("network.http.accept-encoding", "gzip, deflate, compress;q=0.9");
-
 // deactivate mailcap support, it breaks Gnome-based helper apps
 pref("helpers.global_mailcap_file", "");
 pref("helpers.private_mailcap_file", "");
@@ -97,3 +108,22 @@ pref("font.size.variable.zh-HK", 11);
 pref("font.size.variable.tr", 11);
 pref("font.size.variable.x-unicode", 11);
 pref("font.size.variable.x-western", 11);
+
+
+// External protocols
+pref("network.protocol-handler.external.news", true);
+pref("network.protocol-handler.external.mailto", true);
+pref("network.protocol-handler.external.irc", true);
+pref("network.protocol-handler.external.webcal", true);
+
+// Allow myportal file: links (moz 1.8a5 up)
+user_pref("capability.policy.policynames", "myportalok");
+user_pref("capability.policy.default.checkloaduri.enabled", "noAccess");
+user_pref("capability.policy.myportalok.sites", "myportal:");
+user_pref("capability.policy.myportalok.checkloaduri.enabled", "allAccess");
+
+// disable usless security warnings
+pref("security.warn_entering_secure", false);
+pref("security.warn_entering_secure.show_once", true);
+pref("security.warn_leaving_secure", false);
+pref("security.warn_submit_insecure", false);
