@@ -28,12 +28,12 @@ G_BEGIN_DECLS
 
 typedef struct GaleonEmbedEventClass GaleonEmbedEventClass;
 
-#define GALEON_EMBED_EVENT_TYPE             (galeon_embed_event_get_type ())
-#define GALEON_EMBED_EVENT(obj)             (GTK_CHECK_CAST ((obj), GALEON_EMBED_EVENT_TYPE, GaleonEmbedEvent))
+#define GALEON_TYPE_EMBED_EVENT             (galeon_embed_event_get_type ())
+#define GALEON_EMBED_EVENT(obj)             (GTK_CHECK_CAST ((obj), GALEON_TYPE_EMBED_EVENT, GaleonEmbedEvent))
 #define GALEON_EMBED_EVENT_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), GALEON_PERSIST_SHELL, GaleonEmbedEventClass))
-#define IS_GALEON_EMBED_EVENT(obj)          (GTK_CHECK_TYPE ((obj), GALEON_EMBED_EVENT_TYPE))
-#define IS_GALEON_EMBED_EVENT_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GALEON_EMBED_EVENT))
-#define GALEON_EMBED_EVENT_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GALEON_EMBED_SHELL_TYPE, GaleonEmbedEventClass))
+#define GALEON_IS_EMBED_EVENT(obj)          (GTK_CHECK_TYPE ((obj), GALEON_TYPE_EMBED_EVENT))
+#define GALEON_IS_EMBED_EVENT_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), GALEON_EMBED_EVENT))
+#define GALEON_EMBED_EVENT_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GALEON_TYPE_EMBED_SHELL, GaleonEmbedEventClass))
 
 typedef struct GaleonEmbedEvent GaleonEmbedEvent;
 typedef struct GaleonEmbedEventPrivate GaleonEmbedEventPrivate;
@@ -58,9 +58,10 @@ struct GaleonEmbedEvent
 
 	/* Public to the embed implementations */
 	guint modifier;
-	guint mouse_button;
 	guint context;
-	guint mouse_x, mouse_y;
+	guint x, y;
+	guint mouse_button;
+	guint keycode;
 };
 
 struct GaleonEmbedEventClass
@@ -72,25 +73,21 @@ GType             galeon_embed_event_get_type    	(void);
 
 GaleonEmbedEvent *galeon_embed_event_new         	(void);
 
-gresult		  galeon_embed_event_get_modifier	(GaleonEmbedEvent *event,
-							 guint *modifer);
+guint		  galeon_embed_event_get_modifier	(GaleonEmbedEvent *event);
 
-gresult		  galeon_embed_event_get_mouse_button	(GaleonEmbedEvent *event,
-							 guint *mouse_button);
+guint		  galeon_embed_event_get_mouse_button	(GaleonEmbedEvent *event);
 
-gresult		  galeon_embed_event_get_mouse_coords	(GaleonEmbedEvent *event,
+void		  galeon_embed_event_get_coords		(GaleonEmbedEvent *event,
 							 guint *x, guint *y);
 
-gresult		  galeon_embed_event_get_context 	(GaleonEmbedEvent *event,
-						  	 EmbedEventContext *context);
+EmbedEventContext  galeon_embed_event_get_context 	(GaleonEmbedEvent *event);
 
 void		  galeon_embed_event_set_property 	(GaleonEmbedEvent *event,
 							 const char *name,
 							 GValue *value);
 
-void		  galeon_embed_event_get_property	(GaleonEmbedEvent *event,
-							 const char *name,
-							 const GValue **value);
+const GValue *	  galeon_embed_event_get_property	(GaleonEmbedEvent *event,
+							 const char *name);
 
 gboolean	  galeon_embed_event_has_property	(GaleonEmbedEvent *event,
 							 const char *name);
