@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: egg-editable-toolbar.c,v 1.6 2004/04/01 22:33:56 crispin Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,6 +24,7 @@
 
 #include "egg-editable-toolbar.h"
 #include "egg-toolbars-model.h"
+#include "egg-toolbar-editor.h"
 
 #include <gtk/gtkvseparator.h>
 #include <gtk/gtkiconfactory.h>
@@ -47,11 +48,6 @@ static void egg_editable_toolbar_finalize	(GObject *object);
 
 #define MIN_TOOLBAR_HEIGHT 20
 
-static GtkTargetEntry source_drag_types[] = {
-  {EGG_TOOLBAR_ITEM_TYPE, GTK_TARGET_SAME_APP, 0},
-};
-static int n_source_drag_types = G_N_ELEMENTS (source_drag_types);
-
 static GtkTargetEntry dest_drag_types[] = {
   {EGG_TOOLBAR_ITEM_TYPE, GTK_TARGET_SAME_APP, 0},
 };
@@ -61,7 +57,7 @@ enum
 {
   PROP_0,
   PROP_TOOLBARS_MODEL,
-  PROP_MENU_MERGE
+  PROP_UI_MANAGER
 };
 
 enum
@@ -78,10 +74,11 @@ static GObjectClass *parent_class = NULL;
 
 struct EggEditableToolbarPrivate
 {
-  GtkUIManager *merge;
+  GtkUIManager *manager;
   EggToolbarsModel *model;
   gboolean edit_mode;
   GtkWidget *selected_toolbar;
+  GtkToolItem *fixed;
   GtkWidget *fixed_toolbar;
 
   gboolean pending;
@@ -180,9 +177,8 @@ find_action (EggEditableToolbar *t,
   GList *l;
   GtkAction *action = NULL;
 
-  l = gtk_ui_manager_get_action_groups (t->priv->merge);
+  l = gtk_ui_manager_get_action_groups (t->priv->manager);
 
-  g_return_val_if_fail (EGG_IS_EDITABLE_TOOLBAR (t), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
   for (; l != NULL; l = l->next)
@@ -238,24 +234,27 @@ drag_data_get_cb (GtkWidget          *widget,
 		  guint32             time,
 		  EggEditableToolbar *etoolbar)
 {
-  GtkAction *action;
-  const char *target;
+  const char *id, *type;
+  char *target;
 
   g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (etoolbar));
 
-  action = GTK_ACTION (g_object_get_data (G_OBJECT (widget), "gtk-action"));
-
-  if (action)
+  type = g_object_get_data (G_OBJECT (widget), "type");
+  id = g_object_get_data (G_OBJECT (widget), "id");
+  if (strcmp (id, "separator") == 0)
     {
-      target = gtk_action_get_name (action);
+      target = g_strdup (id);
     }
   else
     {
-      target = "separator";
+      target = egg_toolbars_model_get_item_data (etoolbar->priv->model,
+						 type, id);
     }
 
   gtk_selection_data_set (selection_data,
 			  selection_data->target, 8, target, strlen (target));
+
+  g_free (target);
 }
 
 static void
@@ -285,19 +284,30 @@ unset_drag_cursor (GtkWidget *widget)
 }
 
 static void
-set_item_drag_source (GtkWidget *item,
-		      GtkAction *action,
-		      gboolean   is_separator)
+set_item_drag_source (EggToolbarsModel *model,
+		      GtkWidget        *item,
+		      GtkAction        *action,
+		      gboolean          is_separator,
+		      const char       *type)
 {
+  GtkTargetEntry target_entry;
+  const char *id;
+
+  target_entry.target = (char *)type;
+  target_entry.flags = GTK_TARGET_SAME_APP;
+  target_entry.info = 0;
+
   gtk_drag_source_set (item, GDK_BUTTON1_MASK,
-		       source_drag_types, n_source_drag_types,
-		       GDK_ACTION_MOVE);
+                       &target_entry, 1,
+                       GDK_ACTION_MOVE);
 
   if (is_separator)
     {
       GtkWidget *icon;
       GdkPixbuf *pixbuf;
 
+      id = "separator";
+
       icon = _egg_editable_toolbar_new_separator_image ();
       pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (icon));
       gtk_drag_source_set_icon_pixbuf (item, pixbuf);
@@ -308,6 +318,8 @@ set_item_drag_source (GtkWidget *item,
       GValue value = { 0, };
       GdkPixbuf *pixbuf;
 
+      id = gtk_action_get_name (action);
+
       g_value_init (&value, G_TYPE_STRING);
       g_object_get_property (G_OBJECT (action), "stock_id", &value);
       stock_id = g_value_get_string (&value);
@@ -328,11 +340,17 @@ set_item_drag_source (GtkWidget *item,
 
       g_value_unset (&value);
     }
+
+    g_object_set_data_full (G_OBJECT (item), "id",
+                            g_strdup (id), g_free);
+    g_object_set_data_full (G_OBJECT (item), "type",
+                            g_strdup (type), g_free);
 }
 
 static GtkWidget *
 create_item_from_action (EggEditableToolbar *t,
 			 const char *action_name,
+			 const char *type,
 			 gboolean is_separator,
 			 GtkAction **ret_action)
 {
@@ -346,6 +364,8 @@ create_item_from_action (EggEditableToolbar *t,
     }
   else
     {
+      g_return_val_if_fail (action_name != NULL, NULL);
+
       g_signal_emit (G_OBJECT (t), egg_editable_toolbar_signals[ACTION_REQUEST],
 		     0, action_name);
 
@@ -375,7 +395,8 @@ create_item_from_action (EggEditableToolbar *t,
     {
       set_drag_cursor (item);
       gtk_widget_set_sensitive (item, TRUE);
-      set_item_drag_source (item, action, is_separator);
+      set_item_drag_source (t->priv->model, item, action,
+			    is_separator, type);
       gtk_tool_item_set_use_drag_window (GTK_TOOL_ITEM (item), TRUE);
     }
 
@@ -394,12 +415,13 @@ create_item (EggEditableToolbar *t,
 	     int                 position,
              GtkAction         **ret_action)
 {
-  const char *action_name;
+  const char *action_name, *type;
   gboolean is_separator;
 
-  action_name = egg_toolbars_model_item_nth
-		(model, toolbar_position, position, &is_separator);
-  return create_item_from_action (t, action_name, is_separator, ret_action);
+  egg_toolbars_model_item_nth (model, toolbar_position, position,
+			       &is_separator, &action_name, &type);
+  return create_item_from_action (t, action_name, type,
+				  is_separator, ret_action);
 }
 
 static gboolean
@@ -449,7 +471,8 @@ drag_data_received_cb (GtkWidget          *widget,
     {
       etoolbar->priv->pending = FALSE;
       etoolbar->priv->dragged_item =
-        create_item_from_action (etoolbar, id, data_is_separator (id), NULL);
+        create_item_from_action (etoolbar, id, type,
+				 data_is_separator (id), NULL);
       g_object_ref (etoolbar->priv->dragged_item);
       gtk_object_sink (GTK_OBJECT (etoolbar->priv->dragged_item));
     }
@@ -578,6 +601,33 @@ toolbar_drag_motion_cb (GtkWidget          *widget,
   int index;
   GtkToolbar *toolbar = GTK_TOOLBAR (widget);
   GtkToolItem *item;
+  GtkWidget *source;
+
+  source = gtk_drag_get_source_widget (context);
+  if (source)
+    {
+      EggTbModelFlags flags;
+      int pos;
+      gboolean is_item;
+
+      pos = get_toolbar_position (etoolbar, widget);
+      flags = egg_toolbars_model_get_flags (etoolbar->priv->model, pos);
+
+      is_item = etoolbar->priv->edit_mode &&
+		(gtk_widget_get_ancestor (source, EGG_TYPE_EDITABLE_TOOLBAR) ||
+		 gtk_widget_get_ancestor (source, EGG_TYPE_TOOLBAR_EDITOR));
+
+      if ((flags & EGG_TB_MODEL_ACCEPT_ITEMS_ONLY) && !is_item)
+        {
+          gdk_drag_status (context, 0, time);
+          return FALSE;
+        }
+
+      if (gtk_widget_is_ancestor (source, widget))
+        {
+          context->suggested_action = GDK_ACTION_MOVE;
+        }
+    }
 
   target = gtk_drag_dest_find_target (widget, context, NULL);
   if (target == GDK_NONE)
@@ -600,7 +650,8 @@ toolbar_drag_motion_cb (GtkWidget          *widget,
       gtk_drag_get_data (widget, context, target, time);
     }
 
-  if (etoolbar->priv->dragged_item != NULL)
+  if (etoolbar->priv->dragged_item != NULL &&
+      etoolbar->priv->edit_mode)
     {
       item = GTK_TOOL_ITEM (etoolbar->priv->dragged_item);
 
@@ -841,27 +892,6 @@ item_removed_cb (EggToolbarsModel   *model,
     }
 }
 
-static void
-toolbars_clean (EggEditableToolbar *t)
-{
-  GList *children, *l;
-
-  if (t->priv->fixed_toolbar)
-    {
-       unset_fixed_style (t);
-       unparent_fixed (t);
-    }
-
-  children = gtk_container_get_children (GTK_CONTAINER (t));
-
-  for (l = children; l != NULL; l = l->next)
-    {
-      gtk_widget_destroy (GTK_WIDGET (l->data));
-    }
-
-  g_list_free (children);
-}
-
 static void
 egg_editable_toolbar_construct (EggEditableToolbar *t)
 {
@@ -869,6 +899,7 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
   EggToolbarsModel *model = t->priv->model;
 
   g_return_if_fail (model != NULL);
+  g_return_if_fail (t->priv->manager != NULL);
 
   n_toolbars = egg_toolbars_model_n_toolbars (model);
 
@@ -922,74 +953,86 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
         }
     }
 
-    update_fixed (t);
+  update_fixed (t);
 }
 
 static void
-egg_editable_toolbar_set_model (EggEditableToolbar *t,
+egg_editable_toolbar_disconnect_model (EggEditableToolbar *toolbar)
+{
+  EggToolbarsModel *model = toolbar->priv->model;
+
+  g_signal_handlers_disconnect_by_func
+    (model, G_CALLBACK (item_added_cb), toolbar);
+  g_signal_handlers_disconnect_by_func
+    (model, G_CALLBACK (item_removed_cb), toolbar);
+  g_signal_handlers_disconnect_by_func
+    (model, G_CALLBACK (toolbar_added_cb), toolbar);
+  g_signal_handlers_disconnect_by_func
+    (model, G_CALLBACK (toolbar_removed_cb), toolbar);
+  g_signal_handlers_disconnect_by_func
+    (model, G_CALLBACK (toolbar_changed_cb), toolbar);
+}
+
+static void
+egg_editable_toolbar_deconstruct (EggEditableToolbar *toolbar)
+{
+  EggToolbarsModel *model = toolbar->priv->model;
+  GList *children;
+
+  g_return_if_fail (model != NULL);
+
+  if (toolbar->priv->fixed_toolbar)
+    {
+       unset_fixed_style (toolbar);
+       unparent_fixed (toolbar);
+    }
+
+  children = gtk_container_get_children (GTK_CONTAINER (toolbar));
+  g_list_foreach (children, (GFunc) gtk_widget_destroy, NULL);
+  g_list_free (children);
+}
+
+void
+egg_editable_toolbar_set_model (EggEditableToolbar *toolbar,
 				EggToolbarsModel   *model)
 {
   g_return_if_fail (EGG_IS_TOOLBARS_MODEL (model));
-  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (t));
+  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (toolbar));
+  g_return_if_fail (toolbar->priv->manager);
 
-  if (t->priv->model == model) return;
+  if (toolbar->priv->model == model) return;
 
-  if (t->priv->model)
+  if (toolbar->priv->model)
     {
-       g_signal_handlers_disconnect_by_func
-         (t->priv->model, G_CALLBACK (item_added_cb), t);
-       g_signal_handlers_disconnect_by_func
-         (t->priv->model, G_CALLBACK (item_removed_cb), t);
-       g_signal_handlers_disconnect_by_func
-         (t->priv->model, G_CALLBACK (toolbar_added_cb), t);
-       g_signal_handlers_disconnect_by_func
-         (t->priv->model, G_CALLBACK (toolbar_removed_cb), t);
-       g_signal_handlers_disconnect_by_func
-         (t->priv->model, G_CALLBACK (toolbar_changed_cb), t);
+      egg_editable_toolbar_disconnect_model (toolbar);
+      egg_editable_toolbar_deconstruct (toolbar);
 
-       g_object_unref (t->priv->model);
+      g_object_unref (toolbar->priv->model);
     }
 
-  t->priv->model = g_object_ref (model);
+  toolbar->priv->model = g_object_ref (model);
 
-  toolbars_clean (t);
-  if (t->priv->merge != NULL)
-    {
-      egg_editable_toolbar_construct (t);
-    }
+  egg_editable_toolbar_construct (toolbar);
 
-  g_signal_connect_object (model, "item_added",
-			   G_CALLBACK (item_added_cb), t, 0);
-  g_signal_connect_object (model, "item_removed",
-			   G_CALLBACK (item_removed_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_added",
-			   G_CALLBACK (toolbar_added_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_removed",
-			   G_CALLBACK (toolbar_removed_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_changed",
-			   G_CALLBACK (toolbar_changed_cb), t, 0);
+  g_signal_connect (model, "item_added",
+		    G_CALLBACK (item_added_cb), toolbar);
+  g_signal_connect (model, "item_removed",
+		    G_CALLBACK (item_removed_cb), toolbar);
+  g_signal_connect (model, "toolbar_added",
+		    G_CALLBACK (toolbar_added_cb), toolbar);
+  g_signal_connect (model, "toolbar_removed",
+		    G_CALLBACK (toolbar_removed_cb), toolbar);
+  g_signal_connect (model, "toolbar_changed",
+		    G_CALLBACK (toolbar_changed_cb), toolbar);
 }
 
 static void
-egg_editable_toolbar_set_merge (EggEditableToolbar *t,
-				GtkUIManager       *merge)
+egg_editable_toolbar_set_ui_manager (EggEditableToolbar *t,
+				     GtkUIManager       *manager)
 {
-  g_return_if_fail (GTK_IS_UI_MANAGER (merge));
-  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (t));
+  g_return_if_fail (GTK_IS_UI_MANAGER (manager));
 
-  if (t->priv->merge == merge) return;
-
-  if (t->priv->merge)
-    {
-      g_object_unref (t->priv->merge);
-    }
-
-  t->priv->merge = g_object_ref (merge);
-
-  if (t->priv->model != NULL)
-    {
-      egg_editable_toolbar_construct (t);
-    }
+  t->priv->manager = g_object_ref (manager);
 }
 
 static void
@@ -1002,12 +1045,15 @@ egg_editable_toolbar_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_MENU_MERGE:
-      egg_editable_toolbar_set_merge (t, g_value_get_object (value));
+    case PROP_UI_MANAGER:
+      egg_editable_toolbar_set_ui_manager (t, g_value_get_object (value));
       break;
     case PROP_TOOLBARS_MODEL:
       egg_editable_toolbar_set_model (t, g_value_get_object (value));
       break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -1021,12 +1067,15 @@ egg_editable_toolbar_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_MENU_MERGE:
-      g_value_set_object (value, t->priv->merge);
+    case PROP_UI_MANAGER:
+      g_value_set_object (value, t->priv->manager);
       break;
     case PROP_TOOLBARS_MODEL:
       g_value_set_object (value, t->priv->model);
       break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -1050,16 +1099,16 @@ egg_editable_toolbar_class_init (EggEditableToolbarClass *klass)
 		  G_TYPE_NONE, 1, G_TYPE_STRING);
 
   g_object_class_install_property (object_class,
-				   PROP_MENU_MERGE,
-				   g_param_spec_object ("MenuMerge",
-							"MenuMerge",
-							"Menu merge",
+				   PROP_UI_MANAGER,
+				   g_param_spec_object ("ui-manager",
+							"UI-Mmanager",
+							"UI Manager",
 							GTK_TYPE_UI_MANAGER,
 							G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
 				   PROP_TOOLBARS_MODEL,
-				   g_param_spec_object ("ToolbarsModel",
-							"ToolbarsModel",
+				   g_param_spec_object ("model",
+							"Model",
 							"Toolbars Model",
 							EGG_TYPE_TOOLBARS_MODEL,
 							G_PARAM_READWRITE));
@@ -1071,10 +1120,6 @@ static void
 egg_editable_toolbar_init (EggEditableToolbar *t)
 {
   t->priv = EGG_EDITABLE_TOOLBAR_GET_PRIVATE (t);
-
-  t->priv->merge = NULL;
-  t->priv->edit_mode = FALSE;
-  t->priv->dragged_item = NULL;
 }
 
 static void
@@ -1087,13 +1132,14 @@ egg_editable_toolbar_finalize (GObject *object)
       g_object_unref (t->priv->fixed_toolbar);
     }
 
-  if (t->priv->merge)
+  if (t->priv->manager)
     {
-      g_object_unref (t->priv->merge);
+      g_object_unref (t->priv->manager);
     }
 
   if (t->priv->model)
     {
+      egg_editable_toolbar_disconnect_model (t);
       g_object_unref (t->priv->model);
     }
 
@@ -1101,12 +1147,20 @@ egg_editable_toolbar_finalize (GObject *object)
 }
 
 GtkWidget *
-egg_editable_toolbar_new (GtkUIManager     *merge,
-			  EggToolbarsModel *model)
+egg_editable_toolbar_new (GtkUIManager     *manager)
 {
   return GTK_WIDGET (g_object_new (EGG_TYPE_EDITABLE_TOOLBAR,
-				   "ToolbarsModel", model,
-				   "MenuMerge", merge,
+				   "ui-manager", manager,
+				   NULL));
+}
+
+GtkWidget *
+egg_editable_toolbar_new_with_model (GtkUIManager     *manager,
+				     EggToolbarsModel *model)
+{
+  return GTK_WIDGET (g_object_new (EGG_TYPE_EDITABLE_TOOLBAR,
+				   "ui-manager", manager,
+				   "model", model,
 				   NULL));
 }
 
@@ -1134,13 +1188,12 @@ egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
       for (l = 0; l < n_items; l++)
         {
 	  GtkToolItem *item;
-	  const char *action_name;
+	  const char *action_name, *type;
           gboolean is_separator;
-	  GtkAction *action;
+	  GtkAction *action = NULL;
 
-          action_name = egg_toolbars_model_item_nth
-		(etoolbar->priv->model, i, l,
-		 &is_separator);
+          egg_toolbars_model_item_nth (etoolbar->priv->model, i, l,
+		 		       &is_separator, &action_name, &type);
 	  action = find_action (etoolbar, action_name);
 
 	  item = gtk_toolbar_get_nth_item (GTK_TOOLBAR (toolbar), l);
@@ -1150,7 +1203,8 @@ egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
 	    {
               set_drag_cursor (GTK_WIDGET (item));
 	      gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
-              set_item_drag_source (GTK_WIDGET (item), action, is_separator);
+              set_item_drag_source (etoolbar->priv->model, GTK_WIDGET (item),
+				    action, is_separator, type);
             }
 	  else
             {
@@ -1210,26 +1264,37 @@ egg_editable_toolbar_hide (EggEditableToolbar *etoolbar,
     }
 }
 
-GtkToolbar *
-egg_editable_toolbar_set_fixed (EggEditableToolbar *etoolbar,
-				GtkWidget          *fixed)
+void
+egg_editable_toolbar_set_fixed (EggEditableToolbar *toolbar,
+				GtkToolItem        *fixed)
 {
-  GtkWidget *fixed_item;
+  g_return_if_fail (EGG_IS_EDITABLE_TOOLBAR (toolbar));
+  g_return_if_fail (!fixed || GTK_IS_TOOL_ITEM (fixed));
 
-  etoolbar->priv->fixed_toolbar = gtk_toolbar_new ();
-  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (etoolbar->priv->fixed_toolbar), FALSE);
-  g_object_ref (etoolbar->priv->fixed_toolbar);
-  gtk_object_sink (GTK_OBJECT (etoolbar->priv->fixed_toolbar));
+  if (!toolbar->priv->fixed_toolbar)
+    {
+      toolbar->priv->fixed_toolbar = gtk_toolbar_new ();
+      gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar->priv->fixed_toolbar), FALSE);
+      g_object_ref (toolbar->priv->fixed_toolbar);
+      gtk_object_sink (GTK_OBJECT (toolbar->priv->fixed_toolbar));
+    }
 
-  fixed_item = GTK_WIDGET (gtk_tool_item_new ());
-  gtk_toolbar_insert (GTK_TOOLBAR (etoolbar->priv->fixed_toolbar),
-                      GTK_TOOL_ITEM (fixed_item), 0);
+  if (toolbar->priv->fixed)
+    {
+      gtk_container_remove (GTK_CONTAINER (toolbar->priv->fixed_toolbar),
+	    		    GTK_WIDGET (toolbar->priv->fixed));
+      g_object_unref (toolbar->priv->fixed);
+    }
 
-  gtk_container_add (GTK_CONTAINER (fixed_item), fixed);
+  toolbar->priv->fixed = fixed;
 
-  update_fixed (etoolbar);
+  if (fixed)
+    {
+      g_object_ref (fixed);
+      gtk_object_sink (GTK_OBJECT (fixed));
 
-  return GTK_TOOLBAR (etoolbar->priv->fixed_toolbar);
+      gtk_toolbar_insert (GTK_TOOLBAR (toolbar->priv->fixed_toolbar), fixed, 0);
+    }
 }
 
 void
@@ -1394,3 +1459,9 @@ _egg_editable_toolbar_new_separator_image (void)
 
   return image;
 }
+
+EggToolbarsModel *
+egg_editable_toolbar_get_model (EggEditableToolbar *etoolbar)
+{
+  return etoolbar->priv->model;
+}
