@@ -10,9 +10,9 @@
 #endif
 
 #include "mozilla-cookie.h"
+#include "GulString.h"
 
 #include <nsICookie.h>
-#include <nsString.h>
 
 #include <ctime>
 #include <glib/gi18n.h>
@@ -21,8 +21,7 @@ CookieInfo *
 mozilla_cookie_to_info (nsICookie *aCookie)
 {
 	CookieInfo *info = g_new0(CookieInfo, 1);
-	nsCAutoString str;
-	PRBool secure;
+	GulCString str;
 	PRUint64 expires;
 
 	(void)aCookie->GetHost (str);
@@ -37,8 +36,7 @@ mozilla_cookie_to_info (nsICookie *aCookie)
 	(void)aCookie->GetPath (str);
 	info->path = g_strdup (str.get());
 
-	(void)aCookie->GetIsSecure(&secure);
-	info->secure = g_strdup (secure ? _("Yes") : _("No"));
+	(void)aCookie->GetIsSecure(&info->secure);
 
 	(void)aCookie->GetExpires(&expires);
 	if (expires == 0)
