@@ -25,6 +25,7 @@
 #include "gul-general.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
+#include "gul-x11.h"
 
 #include <string.h>
 #include <time.h>
@@ -66,6 +67,10 @@ static void session_create_bookmarks (Session *session, const char *filename);
 
 static GObjectClass *parent_class = NULL;
 
+#define SESSION_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       TYPE_SESSION, SessionPrivate))
+
+
 struct SessionPrivate
 {
 	GaleonDialog *crash_dialog;
@@ -154,6 +159,8 @@ session_class_init (SessionClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	g_type_class_add_private (klass, sizeof (SessionPrivate));
 }
 
 static char *
@@ -206,7 +213,7 @@ do_session_resume (Session *session, int type)
 	switch (type)
 	{
 	case RESTORE_SESSION:
-		session_load (session, crashed_session);
+		session_load (session, crashed_session, 0u);
 		break;
 	case RESTORE_AS_BOOKMARKS:
 		session_create_bookmarks (session, crashed_session);
@@ -289,7 +296,7 @@ session_autoresume (Session *session)
 	    && !loaded
 	    && eel_gconf_get_boolean (CONF_GENERAL_ALWAYS_SAVE_SESSION))
 	{
-		session_load (session, saved_session);
+		session_load (session, saved_session, 0u);
 	}
 	
 	g_free (saved_session);
@@ -371,7 +378,7 @@ gnome_session_init (Session *session)
 static void
 session_init (Session *session)
 {
-        session->priv = g_new0 (SessionPrivate, 1);
+        session->priv = SESSION_GET_PRIVATE (session);
 	session->priv->windows = NULL;
 	session->priv->dont_remove_crashed = FALSE;
 	session->priv->session_saved = FALSE;
@@ -447,7 +454,6 @@ session_finalize (GObject *object)
 
 	g_list_free (t->priv->windows);
 	
-        g_free (t->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -464,7 +470,7 @@ session_new (void)
 {
 	Session *t;
 
-	t = SESSION (g_object_new (SESSION_TYPE, NULL));
+	t = SESSION (g_object_new (TYPE_SESSION, NULL));
 
 	g_return_val_if_fail (t->priv != NULL, NULL);
 
@@ -680,13 +686,15 @@ create_new_tab (const char *url, const char *title, GaleonWindow *window)
 static void
 create_new_window (xmlNodePtr child,
 		   const int x, const int y, 
-		   const int width, const int height)
+		   const int width, const int height, gpointer data)
 {
 	GtkWidget *wmain;
 	GaleonWindow *window;
 
 	window = galeon_window_new ();
 	wmain = GTK_WIDGET (window);
+
+	gul_x11_window_update_user_time (wmain, GPOINTER_TO_INT (data));
 	gtk_widget_show (GTK_WIDGET(window));
 	
 	gtk_window_move (GTK_WINDOW(wmain), x, y);
@@ -734,9 +742,10 @@ bookmarks_from_window (xmlNodePtr child,
  **/
 
 void 
-session_load (Session *session, const char *filename)
+session_load (Session *session, const char *filename, guint32 user_time)
 {
-	parse_xml (session, filename, (WindowCb)create_new_window, NULL);
+	parse_xml (session, filename, (WindowCb)create_new_window,
+		   GINT_TO_POINTER(user_time));
 }
 
 GList *
